#include "wobulation.h"
#include <QFileDialog>
#include <QDir>

#define CLSID_DDC1100					"{3F5D1418-3DE0-437A-A834-28E5314D326E}"

#define TANGO_DEFAULT_COMPORT_NAME		"COM1"
#define TANGO_DEFAULT_COMPORT_BAUDRATE	57600

#define LC8_DEFAULT_COMPORT_NAME		"COM3"


Wobulation::Wobulation(QWidget *parent, Qt::WindowFlags flags)
	: QDialog(parent, flags)
{
	ui.setupUi(this);

	int i;
	
	int portNum = ui.uvPortCombo->count();
	for(i=0; i<portNum; i++)
	{
		if(ui.uvPortCombo->itemText(i) == LC8_DEFAULT_COMPORT_NAME)
		{
			ui.uvPortCombo->setCurrentIndex(i);
			break;
		}
	}

	portNum = ui.stagePortCombo->count();
	for(i=0; i<portNum; i++)
	{
		if(ui.stagePortCombo->itemText(i) == TANGO_DEFAULT_COMPORT_NAME)
		{
			ui.stagePortCombo->setCurrentIndex(i);
			break;
		}
	}

	// initialize DMD
	m_pDDC1100 = new DMD(CLSID_DDC1100, this);

	// initialize motorized stage
	m_pTango = new CTango();
	m_pTango->ConnectSimple(1, TANGO_DEFAULT_COMPORT_NAME, TANGO_DEFAULT_COMPORT_BAUDRATE, false);

	connect(ui.stageUp, SIGNAL(pressed()), this, SLOT(slotStageUpBtnPressed()));
	connect(ui.stageDown, SIGNAL(pressed()), this, SLOT(slotStageDownBtnPressed()));
	connect(ui.stageLeft, SIGNAL(pressed()), this, SLOT(slotStageLeftBtnPressed()));
	connect(ui.stageRight, SIGNAL(pressed()), this, SLOT(slotStageRightBtnPressed()));

	// initialize UV source	
	m_pLC8Ctrl = new LC8Control();
	connect(m_pLC8Ctrl, SIGNAL(signalConnected()), this, SLOT(slotUVConncected()));
	connect(m_pLC8Ctrl, SIGNAL(signalIntensity(int)), this, SLOT(slotUVIntensity(int)));
	connect(ui.uvIntensityUp, SIGNAL(pressed()), m_pLC8Ctrl, SLOT(intensityUp()));
	connect(ui.uvIntensityDown, SIGNAL(pressed()), m_pLC8Ctrl, SLOT(intensityDown()));
	connect(ui.uvIntensityUp, SIGNAL(released()), m_pLC8Ctrl, SLOT(intensityStop()));
	connect(ui.uvIntensityDown, SIGNAL(released()), m_pLC8Ctrl, SLOT(intensityStop()));

	m_pLC8Ctrl->connect(LC8_DEFAULT_COMPORT_NAME);

	connect(ui.uvPortSetBtn, SIGNAL(clicked()), this, SLOT(slotUVPortSetBtnClicked()));
	connect(ui.stagePortSetBtn, SIGNAL(clicked()), this, SLOT(slotStagePortSetBtnClicked()));

	connect(ui.listAdd, SIGNAL(clicked()), this, SLOT(slotListAddBtnClicked()));
	connect(ui.listDel, SIGNAL(clicked()), this, SLOT(slotListDelBtnClicked()));
	connect(ui.listClear, SIGNAL(clicked()), this, SLOT(slotListClearBtnClicked()));
	connect(ui.tableWidget, SIGNAL(cellDoubleClicked(int, int)), this, SLOT(slotTableDoubleClicked(int, int)));


	connect(ui.maskBtn1, SIGNAL(clicked()), this, SLOT(slotMaskBtn1Clicked()));
	connect(ui.maskBtn2, SIGNAL(clicked()), this, SLOT(slotMaskBtn2Clicked()));
	
	connect(ui.totalTime, SIGNAL(textEdited(const QString &)), this, SLOT(slotTotalTimeEdited(const QString &)));
	connect(ui.cycle, SIGNAL(textEdited(const QString &)), this, SLOT(slotCycleEdited(const QString &)));
	connect(ui.cycleTime, SIGNAL(textChanged(const QString &)), this, SLOT(slotCycleTimeChanged(const QString &)));
	
	connect(ui.durationMask1, SIGNAL(textEdited(const QString &)), this, SLOT(slotDurationMask1Edited(const QString &)));
	connect(ui.durationMask2, SIGNAL(textEdited(const QString &)), this, SLOT(slotDurationMask2Edited(const QString &)));

	connect(ui.durationSequenceCheck, SIGNAL(toggled(bool)), ui.durationSequenceMask1, SLOT(setEnabled(bool)));
	connect(ui.durationSequenceCheck, SIGNAL(toggled(bool)), ui.durationSequenceMask2, SLOT(setEnabled(bool)));
	connect(ui.durationSequenceCheck, SIGNAL(toggled(bool)), ui.durationSequenceStep, SLOT(setEnabled(bool)));
	connect(ui.durationSequenceCheck, SIGNAL(toggled(bool)), ui.nDurationSequence, SLOT(setEnabled(bool)));
	connect(ui.durationSequenceCheck, SIGNAL(toggled(bool)), ui.interSequenceShiftX, SLOT(setEnabled(bool)));
	connect(ui.durationSequenceCheck, SIGNAL(toggled(bool)), ui.interSequenceShiftY, SLOT(setEnabled(bool)));
	
	connect(ui.durationSequenceMask1, SIGNAL(textEdited(const QString &)), this, SLOT(slotDurationSequenceMask1Edited(const QString &)));
	connect(ui.durationSequenceMask2, SIGNAL(textEdited(const QString &)), this, SLOT(slotDurationSequenceMask2Edited(const QString &)));
	connect(ui.durationSequenceStep, SIGNAL(textEdited(const QString &)), this, SLOT(slotUpdateDurationSequenceNumber()));	
	
	
	connect(ui.runBtn, SIGNAL(clicked()), this, SLOT(slotRunBtnClicked()));
	connect(m_pLC8Ctrl, SIGNAL(signalShutterOpen()), this, SLOT(slotUVShutterOpened()));

	connect(m_pDDC1100, SIGNAL(signalDMDCleared()), this, SLOT(slotDMDCleared()));

	curDir = tr(".");
	m_nBufNumMask1 = m_nBufNumMask2 = -1;
}

Wobulation::~Wobulation()
{

}

void Wobulation::slotUVConncected()
{
	m_pLC8Ctrl->queryIntensity();
}

void Wobulation::slotUVIntensity(int percent)
{
	ui.uvIntensity->setText(QString("%1").arg(percent));
}

void Wobulation::slotStageUpBtnPressed()
{
	if(ui.stageStepLabel->isChecked())
	{
		double nmStep = (ui.stageStep->text()).toDouble();
		double mmStep = nmStep / 1000000;
		m_pTango->MoveRel(0.0, -mmStep, 0.0, 0.0, true);
	}	
}

void Wobulation::slotStageDownBtnPressed()
{
	if(ui.stageStepLabel->isChecked())
	{
		double nmStep = (ui.stageStep->text()).toDouble();
		double mmStep = nmStep / 1000000;
		m_pTango->MoveRel(0.0, mmStep, 0.0, 0.0, true);
	}	
}

void Wobulation::slotStageLeftBtnPressed()
{
	if(ui.stageStepLabel->isChecked())
	{
		double nmStep = (ui.stageStep->text()).toDouble();
		double mmStep = nmStep / 1000000;
		m_pTango->MoveRel(-mmStep, 0.0, 0.0, 0.0, true);
	}	
}

void Wobulation::slotStageRightBtnPressed()
{
	if(ui.stageStepLabel->isChecked())
	{
		double nmStep = (ui.stageStep->text()).toDouble();
		double mmStep = nmStep / 1000000;
		m_pTango->MoveRel(mmStep, 0.0, 0.0, 0.0, true);
	}	
}

void Wobulation::slotUVPortSetBtnClicked()
{
	QString portName = ui.uvPortCombo->currentText();
	m_pLC8Ctrl->disconnect();
	m_pLC8Ctrl->connect(portName);
}

void Wobulation::slotStagePortSetBtnClicked()
{
	QString portName = ui.stagePortCombo->currentText();
	m_pTango->Disconnect();
//	m_pTango->ConnectSimple(1, portName.toAscii().data(), TANGO_DEFAULT_COMPORT_BAUDRATE, false);
	m_pTango->ConnectSimple(1, portName.toLatin1().data(), TANGO_DEFAULT_COMPORT_BAUDRATE, false);
}

void Wobulation::slotListAddBtnClicked()
{
	ItemDlg dlg = ItemDlg();
	if(dlg.exec() == QDialog::Accepted)
	{
		int type = dlg.getActionType();
		QTableWidgetItem *actionItem;
		QTableWidgetItem *parameterItem;

		if(type == 0)
		{
			double exposureTime = dlg.getExposureTime();
			QString maskFile = dlg.getMaskFile();
			
			if(maskFile.isEmpty()) maskFile = tr("blank");

			actionItem = new QTableWidgetItem(tr("Exposure"));
			parameterItem = new QTableWidgetItem(tr("%1ms / %2").arg(exposureTime).arg(maskFile));
			parameterItem->setData(Qt::UserRole, QVariant(exposureTime));
			parameterItem->setData(Qt::UserRole+1, QVariant(maskFile));
		}
		else if(type == 1)
		{
			bool bOrigin;
			double x, y;

			dlg.getMove(&bOrigin, &x, &y);

			actionItem = new QTableWidgetItem(tr("Move"));
			if(bOrigin) parameterItem = new QTableWidgetItem(tr("Origin"));
			else
			{
				parameterItem = new QTableWidgetItem(tr("X=%1nm, Y=%2nm").arg(x).arg(y));
				parameterItem->setData(Qt::UserRole, QVariant(x));
				parameterItem->setData(Qt::UserRole+1, QVariant(y));
			}
		}

		actionItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		parameterItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);

		int row = ui.tableWidget->rowCount();
		ui.tableWidget->setRowCount(row+1);
		ui.tableWidget->setItem(row, 0, actionItem);
		ui.tableWidget->setItem(row, 1, parameterItem);
	}
}

void Wobulation::slotListDelBtnClicked()
{
	int row = ui.tableWidget->currentRow();
	ui.tableWidget->removeRow(row);
}

void Wobulation::slotListClearBtnClicked()
{
	ui.tableWidget->clearContents();
	ui.tableWidget->setRowCount(0);
}

void Wobulation::slotTableDoubleClicked(int row, int column)
{
	QTableWidgetItem *actionItem = ui.tableWidget->item(row, 0);
	QTableWidgetItem *parameterItem = ui.tableWidget->item(row, 1);

	ItemDlg dlg = ItemDlg();

	bool ok;

	QString type = actionItem->text();
	if(type == tr("Exposure"))
	{
		dlg.disableTab(1);
		
		dlg.setExposureTime((parameterItem->data(Qt::UserRole)).toDouble(&ok));

		QString fileName = (parameterItem->data(Qt::UserRole+1)).toString();
		if(fileName != tr("blank"))	dlg.setMaskFile(fileName);

		if(dlg.exec() == QDialog::Accepted)
		{
			double exposureTime = dlg.getExposureTime();
			QString maskFile = dlg.getMaskFile();

			if(maskFile.isEmpty()) maskFile = tr("blank");
			
			parameterItem->setText(tr("%1ms / %2").arg(exposureTime).arg(maskFile));
			parameterItem->setData(Qt::UserRole, QVariant(exposureTime));
			parameterItem->setData(Qt::UserRole+1, QVariant(maskFile));
		}
	}
	else if(type == tr("Move"))
	{
		dlg.disableTab(0);

		bool bOrigin = false;
		double x, y;
		if(parameterItem->text() == tr("Origin")) bOrigin = true;
		else
		{
			x = (parameterItem->data(Qt::UserRole)).toInt(&ok);
			y = (parameterItem->data(Qt::UserRole+1)).toInt(&ok);
		}

		dlg.setMove(bOrigin, x, y);

		if(dlg.exec() == QDialog::Accepted)
		{
			bool bOrigin;
			double x, y;

			dlg.getMove(&bOrigin, &x, &y);

			if(bOrigin) parameterItem->setText(tr("Origin"));
			else 
			{
				parameterItem->setText(tr("X=%1nm, Y=%2nm").arg(x).arg(y));
				parameterItem->setData(Qt::UserRole, QVariant(x));
				parameterItem->setData(Qt::UserRole+1, QVariant(y));
			}
		}
	}
}


void Wobulation::slotMaskBtn1Clicked()
{
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), curDir, tr("Images (*.bmp)"));

	if(fileName.isEmpty() == false)
	{
		curDir = QDir().filePath(fileName);
		ui.maskLabel1->setPixmap(QPixmap(fileName).scaled(256, 192));
		ui.maskName1->setText(fileName);

#ifdef USE_WLP_DMD
		m_strFileNameMask1 = fileName;
#else
		m_nBufNumMask1 = m_pDDC1100->loadToMemBuffer(fileName);
#endif
	}
}

void Wobulation::slotMaskBtn2Clicked()
{
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), curDir, tr("Images (*.bmp)"));

	if(fileName.isEmpty() == false)
	{
		curDir = QDir().filePath(fileName);
		ui.maskLabel2->setPixmap(QPixmap(fileName).scaled(256, 192));
		ui.maskName2->setText(fileName);

#ifdef USE_WLP_DMD
		m_strFileNameMask2 = fileName;
#else
		m_nBufNumMask2 = m_pDDC1100->loadToMemBuffer(fileName);
#endif
	}
}

void Wobulation::slotTotalTimeEdited(const QString &text)
{
	int totalTime = text.toInt();
	int cycle = (ui.cycle->text()).toInt();
	int cycleTime = totalTime / cycle;

	ui.cycleTime->setText(QString("%1").arg(cycleTime));
}

void Wobulation::slotCycleEdited(const QString &text)
{
	int totalTime = (ui.totalTime->text()).toInt();
	int cycle = text.toInt();
	if(cycle <= 0) return;

	int cycleTime = (int)totalTime / cycle;


	ui.cycleTime->setText(QString("%1").arg(cycleTime));
}

void Wobulation::slotCycleTimeChanged(const QString &text)
{
	int cycleTime = text.toInt();
	ui.durationMask1->setText(QString("%1").arg(cycleTime/2));
	ui.durationMask2->setText(QString("%1").arg(cycleTime/2));
	ui.durationSequenceMask1->setText(QString("%1").arg(cycleTime/2));
	ui.durationSequenceMask2->setText(QString("%1").arg(cycleTime/2));
	slotUpdateDurationSequenceNumber();
}

void Wobulation::slotDurationMask1Edited(const QString &text)
{
	int cycleTime = (ui.cycleTime->text()).toInt();
	int duration1 = text.toInt();
	int duration2 = cycleTime - duration1;

	ui.durationMask2->setText(QString("%1").arg(duration2));

	slotUpdateDurationSequenceNumber();
}

void Wobulation::slotDurationMask2Edited(const QString &text)
{
	int cycleTime = (ui.cycleTime->text()).toInt();
	int duration2 = text.toInt();
	int duration1 = cycleTime - duration2;

	ui.durationMask1->setText(QString("%1").arg(duration1));

	slotUpdateDurationSequenceNumber();
}

void Wobulation::slotDurationSequenceMask1Edited(const QString &text)
{
	int cycleTime = (ui.cycleTime->text()).toInt();
	int duration1 = text.toInt();
	int duration2 = cycleTime - duration1;

	ui.durationSequenceMask2->setText(QString("%1").arg(duration2));

	slotUpdateDurationSequenceNumber();
}

void Wobulation::slotDurationSequenceMask2Edited(const QString &text)
{
	int cycleTime = (ui.cycleTime->text()).toInt();
	int duration2 = text.toInt();
	int duration1 = cycleTime - duration2;

	ui.durationSequenceMask1->setText(QString("%1").arg(duration1));

	slotUpdateDurationSequenceNumber();
}

void Wobulation::slotUpdateDurationSequenceNumber()
{
	int sDuration1 = (ui.durationMask1->text()).toInt();
	int eDuration1 = (ui.durationSequenceMask1->text()).toInt();

	int diff = sDuration1 - eDuration1;
	if(diff < 0) diff = -diff;

	int step = (ui.durationSequenceStep->text()).toInt();
	if(step <= 0) return;

	int nSeq = diff / step + 1;

	ui.nDurationSequence->setText(QString("%1").arg(nSeq));

}

bool Wobulation::checkMode1Settings(void)
{
	if(ui.tableWidget->rowCount() <= 0) return false;

	bool ok;

	m_nListRepeatNum = (ui.repeatNum->text()).toInt(&ok);
	if(m_nListRepeatNum<=0 | !ok) return false;

	return true;
}

bool Wobulation::checkMode2Settings(void)
{
#ifdef USE_WLP_DMD
	if(m_strFileNameMask1.isEmpty() | m_strFileNameMask2.isEmpty()) return false;
#else
	if(m_nBufNumMask1==-1 | m_nBufNumMask2==-1) return false;
#endif
	bool ok;

	m_nCycleTime = (ui.cycleTime->text()).toInt(&ok);
	if(m_nCycleTime<=0 | !ok) return false;

	m_nCycle = (ui.cycle->text()).toInt();
	if(m_nCycle<=0  | !ok) return false;

	m_nDurationMask1 = (ui.durationMask1->text()).toInt(&ok);
	if(m_nDurationMask1<0 | !ok) return false;

	m_nDurationMask2 = (ui.durationMask2->text()).toInt(&ok);
	if(m_nDurationMask2<0 | !ok) return false;

	m_nShiftX = (ui.shiftX->text()).toInt(&ok);
	if(!ok) return false;

	m_nShiftY = (ui.shiftY->text()).toInt(&ok);
	if(!ok) return false;
	
	m_bDurationSequenceChecked = ui.durationSequenceCheck->isChecked();
	if(m_bDurationSequenceChecked)
	{
		m_nDurationSequenceStep = (ui.durationSequenceStep->text()).toInt(&ok);
		if(m_nDurationSequenceStep<=0 | !ok) return false;

		m_nDurationSequence = (ui.nDurationSequence->text()).toInt(&ok);
		if(!ok) return false;

		m_nDurationSequenceMask1 = (ui.durationSequenceMask1->text()).toInt(&ok);
		if(m_nDurationSequenceMask1<0 | !ok) return false;

		m_nDurationSequenceMask2 = (ui.durationSequenceMask1->text()).toInt(&ok);
		if(m_nDurationSequenceMask2<0 | !ok) return false;

		m_nInterSequenceShiftX = (ui.interSequenceShiftX->text()).toInt(&ok);
		if(!ok) return false;

		m_nInterSequenceShiftY = (ui.interSequenceShiftY->text()).toInt(&ok);
		if(!ok) return false;

		if(m_nDurationMask1 > m_nDurationSequenceMask1) m_nDurationSequenceStep = -m_nDurationSequenceStep;
	}

	return true;
}
